package KorAP::DefList;
use KorAP::Def;
use KorAP::VirtualCorpus::Group;
use KorAP::VirtualCorpus::Doc;
use File::Spec::Functions qw!catfile!;
use strict;
use warnings;

# Constructor
sub new {
  my $class = shift;
  my %self = @_;

  $self{output} //= '.';

  my $file = $self{file};

  if (ref $file && ref $file eq 'GLOB') {
    $self{file} = '';
    $self{fh} = $file;
    return bless \%self, $class;
  };

  # Open def file
  if (!open($self{fh}, '<' . $file)) {
    warn $file . " can't be opened";
    exit(0);
  };

  return bless \%self, $class;
};


# Parse list file
sub parse {
  my $self = shift;

  while (!eof($self->{fh})) {
    my $line = readline($self->{fh});

    next if $line =~ /^\#/;

    if ($line =~ /^(.+?)\t+\@COPY\@\s*$/) {

      # Take from Def file!
      $self->copy_vc($1);
    }

    elsif ($line =~ /^([^\t]+?)\t+(.*?)\t(.+?)\/?$/) {
      $self->regex_to_vc($1,$2,$3);
      # print $1,': /'.$3.'/',"\n";
    }

    elsif ($line =~ /^\s*$/) {
      # Ignore
    }

    else {
      warn 'Unknown VC definition: ', $line,"\n";
    };
  };

  close($self->{fh});
};

sub copy_vc {
  my $self = shift;
  my $file = shift;

  print "Convert ", $file, " from def-file\n";

  # Open file from copy directory
  my $def = KorAP::Def->new(catfile($self->{copy}, $file));

  # Parse
  return unless $def;
  $def->parse or return;

  # Output
  my $out = catfile($self->{output}, $file . '.json');
  if (open(my $koral, '>' . $out)) {
    print $koral $def->to_string;
    close($koral);
    return;
  };

  warn 'Unable to write file ' . $out;
};

sub regex_to_vc {
  my $self = shift;
  my ($name, $desc, $regex) = @_;

  print "Convert ", $name, " from regex\n";

  my $vc = from_regex($regex, $name, $desc);

  # Output
  my $out = catfile($self->{output}, $name . '.json');
  if (open(my $koral, '>' . $out)) {
    print $koral $vc->to_string;
    close($koral);
    return;
  };

  warn 'Unable to write file ' . $out;
};

sub from_regex {
  my $regex = shift;
  my $name = shift;
  my $desc = pop;

  my $vc;

  # Group with negation
  if ($regex =~ m!^([^~]+?)~(.+?)$!) {
    $vc = KorAP::VirtualCorpus::Group->new;

    my $pos = KorAP::VirtualCorpus::Doc->new;
    _regex_to_doc($pos, $1);

    $vc->add('and', $pos);

    my $neg = KorAP::VirtualCorpus::Doc->new;
    _regex_to_doc($neg, $2);

    $neg->match('ne');
    $vc->joint($neg);
  }

  # Simle doc
  else {
    $vc = KorAP::VirtualCorpus::Doc->new;
    _regex_to_doc($vc, $regex);
  };

  $vc->name($name) if $name;
  $vc->comment('desc:' . $vc->equote($desc)) if $desc;

  return $vc;
};

sub _regex_to_doc {
  my ($vc, $regex) = @_;

  if ($regex =~ m!^([^/]+)\/?$!) {
    $vc->key('corpusSigle');
    $vc->value($1);
    my $value = $1;
    if ($1 !~ /^[a-zA-Z0-9]+?$/) {
      $vc->type('regex');
    };
    return;
  }

  $vc->key('docSigle');
  $vc->value($regex);
  $vc->type('regex');
}

1;
